/*2004 Timo Latvala <timo.latvala@hut.fi>. See the file LGPL-2.1 for details*/
/**A hashtable for (unsigned, unsigned) pairs*/

#include <stdlib.h>
#include "bmc/bmcHash.h"
#include "utils/utils.h"

/**Constant declarations*/

#define HASH_TABLE_SIZE 127
#define HASH_MAX_DENSITY 0.5


/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/
static int table_hash_fun ARGS((int, int));
static int find ARGS((hashPtr table, int));

/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Create a new hash_table*/

hashPtr Bmc_Hash_new_htable()
{
  int i;
  hashPtr hash_table = (hashPtr) malloc(sizeof(struct htable));
  hash_table->alloc=HASH_TABLE_SIZE;
  hash_table->occupied=0;
  hash_table->table = (struct table_pair *) malloc(sizeof(struct table_pair) * hash_table->alloc);
  if (hash_table->table == NULL) {
    fprintf(stderr, "new_assoc: Out of Memory\n");
    exit(1);
  }
  for(i=hash_table->alloc; i--; ) {
    hash_table->table[i].key=0;
  }
  return hash_table;
}

/**Find an element in the table
 * @return BMC_HASH_NOTFOUND if the key is not present 
 */

int Bmc_Hash_find(hashPtr table, int key) 
{ 
  nusmv_assert(key>=0);
  int i=find(table, key+1);
  if(table->table[i].key) 
    return table->table[i].data;
  return BMC_HASH_NOTFOUND;
}

/**@return the number of occupied slots*/
unsigned Bmc_Hash_size(hashPtr hash) 
{
  return hash->occupied;
}

/**insert a new element in the table*/
void Bmc_Hash_insert (hashPtr table, int key, int data)
{
  nusmv_assert(key>=0 && data>=0);
  int i=find(table, key+1);
  if(table->table[i].key)
    return; /**The key already is in the table*/
  if( (table->occupied+1)/table->alloc > 0.5 ) { /*a rehash is needed*/    
    unsigned j;
    struct table_pair *temp= table->table;
    table->alloc=(table->alloc)*2;
    table->table= (struct table_pair *) 
      malloc(sizeof(struct table_pair) * table->alloc);
    nusmv_assert(table->table != NULL);
    
    /**reset new table*/
    for(j=table->alloc; j--; ) {
      (table->table)[j].key = 0;
    }
    
    /**copy the old table to the one*/
    for( j=(table->alloc)/2; j--; ) {
      if(temp[j].key == 0) /**Empty slot*/
	continue;
      int index=find(table, temp[j].key);
      nusmv_assert( (table->table)[index].key == 0 );
      (table->table)[index].data=temp[j].data;
      (table->table)[index].key=temp[j].key;
    }
    free(temp);
    Bmc_Hash_insert(table, key, data);
    return;
  }
  nusmv_assert((table->table)[i].key == 0);
  (table->table)[i].key=key+1;
  (table->table)[i].data=data;
  (table->occupied)++;
  return;
}

/**delete the table*/

void Bmc_Hash_delete_table(hashPtr hash)
{
  free(hash->table);
  free(hash);
}

/*---------------------------------------------------------------------------*/
/* Definition of static functions                                            */
/*---------------------------------------------------------------------------*/

static int table_hash_fun(int key, int size)
{ return((int)(key) % size); }

/**@return index of key, a free index if the element is not in the table*/
static int find(hashPtr table, int key)
{
  int hash=table_hash_fun(key, table->alloc);
  int i;
  for(i=hash; ; ) {
    if((table->table)[i].key) {
      if(key==(table->table)[i].key)
	return i;
    }
    else {
      break;
    }
    i=(i+1)%(table->alloc);
  }
  return i;
}
