/**CHeaderFile*****************************************************************

  FileName    [bmcPsl.h]

  PackageName [bmc]

  Synopsis    [Public interface for generating PSL SAT problems]

  Description []

  SeeAlso     []

  Author      [Timo Latvala]

  Copyright   [
  This file is part of the ``bmc'' package of NuSMV version 2.
  Copyright (C) 2000-2001 by ITC-irst and University of Trento.

  NuSMV version 2 is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  NuSMV version 2 is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.

  For more information of NuSMV see <http://nusmv.irst.itc.it>
  or email to <nusmv-users@irst.itc.it>.
  Please report bugs to <nusmv-users@irst.itc.it>.

  To contact the NuSMV development board, email to <nusmv@irst.itc.it>. ]

  Revision    [$Id: bmcPartitionMap.h,v 1.4 2006/01/17 23:37:42 timo Exp $]

******************************************************************************/

#ifndef _BMC_PARTITIONMAP__H
#define _BMC_PARTITIONMAP__H

#include "utils/utils.h"
#include "sugarInt.h"
#include "bmcHash.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Structure declarations                                                    */
/*---------------------------------------------------------------------------*/

struct BmcPartitionMap {
  const Sugar_Automaton_t *aut; /*The automaton that the map partitions*/
  int numPartitions; /*The number of partitions in the map (the range of the map)*/
  int *map; /*Array mapping vertex ids to partition ids (0..numPartitions-1)*/
  int *partsize; /*Map vertex ids to partition sizes (0..size-1)*/
  int *vertexmap; /*Mapping consequtive vertex ids (0..size-1) to the automaton's vertex ids*/ 
  hashPtr idToVertexId; /*Map automata vertex ids to consequtive vertex ids*/
};


/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/



/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN struct BmcPartitionMap*  
Bmc_PartitionMapCreate ARGS((const Sugar_Automaton_t *aut));

EXTERN void
Bmc_PartitionMapDelete ARGS((struct BmcPartitionMap *map));

EXTERN void
Bmc_PartitionMapInit ARGS((struct BmcPartitionMap *map));

EXTERN int
Bmc_PartitionMapGet ARGS((struct BmcPartitionMap *map, int vertex_id));

EXTERN int
Bmc_PartitionMapGet_AutId ARGS((struct BmcPartitionMap *map, int aut_id));

EXTERN void
Bmc_PartitionMapSet ARGS((struct BmcPartitionMap *map, int vertex_id, int partitionId));

EXTERN int
Bmc_PartitionMap_AutToId ARGS((struct BmcPartitionMap *map, int vertex_id));

EXTERN int
Bmc_PartitionMap_IdToAut ARGS((struct BmcPartitionMap *map, int aut_id));

EXTERN int
Bmc_PartitionMapPartSize ARGS((struct BmcPartitionMap *map, int vertex_id));

EXTERN void
Bmc_PartitionMapPartSizeSet ARGS((struct BmcPartitionMap *map, int vertex_id, int size));

/**AutomaticEnd***************************************************************/

#endif /* _BMC_PARTITIONMAP__H */

