/***************************************************************  -*-C-*-  ***/

%{
/**CFile***********************************************************************

  FileName    [aba_input.l]

  PackageName [parser.aba]

  Synopsis    [Lexical analyzer for the ABA specification input language]

  SeeAlso     [aba_grammar.y]

  Author      [Tommi Junttila]

  Copyright   [Copyright (C) 2006 by Tommi Junttila. Released under GNU LGPL]

******************************************************************************/

#include <stdio.h>
#include <errno.h>

#include "utils/utils.h"
#include "utils/array.h"
#include "bmc/sugar.h"
#include "bmc/sugarInt.h"
#include "aba_grammar.h"

static char rcsid[] UTIL_UNUSED = "$Id: aba_input.l,v 1.1 2006/05/05 12:13:47 tjunttil Exp $";

static int aba_skip_comment ARGS((void));
EXTERN void aba_yyerror ARGS((char* s, ...));


#define YY_INPUT(buf, result, max_size) \
	if ( YY_CURRENT_BUFFER->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for (n=0; n<max_size && \
		     (c = getc(aba_yyin)) != EOF && \
                     c!=' ' && c!='\n' && c!='\t' && c!='\r' && c!='\f'; ++n ) \
			buf[n] = (char) c; \
		if (c==' ' || c=='\n' || c=='\t' || c=='\r' || c=='\f') buf[n++] = (char) c; \
		if (c == EOF && ferror(aba_yyin)) YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, aba_yyin))==0 && ferror(aba_yyin)) \
			{ \
			if(errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(aba_yyin); \
			} \
		}


%}


%option noyywrap
%option yylineno
%option always-interactive

ABA_TOK_SPACE               [ \n\t\r\f]

ABA_TOK_DIGIT               [0-9]
ABA_TOK_ALPHA               [A-Za-z_]
ABA_TOK_ALPHANUM            [A-Za-z_0-9-]

ABA_TOK_COMMENT             "--"
TKTRUE                      "TRUE"
TKFALSE                     "FALSE"

TKABA                       "aba"
TKSTATE                     "state"
TKACCEPTING                 "accepting"
TKINITIAL                   "initial"

TKAMPERSAND                 "&"

TKLP                        "("
TKRP                        ")"
TKLC                        "{"
TKRC                        "}"
TKLB                        "["
TKRB                        "]"
TKCOMMA                     ","
TKQUESTIONMARK              "?"
TKCOLON                     ":"
TKSEMI                      ";"
TKDOT                       "."

TKBANG                      "!"
TKTILDE                     "~"



%%

{ABA_TOK_SPACE}*         ;
{ABA_TOK_COMMENT}        { aba_skip_comment(); }
{TKTRUE}                 { return TKTRUE; }
{TKFALSE}                { return TKFALSE; }

{TKABA}                  { return TKABA; }
{TKSTATE}                { return TKSTATE; }
{TKINITIAL}              { return TKINITIAL; }
{TKACCEPTING}            { return TKACCEPTING; }

{TKAMPERSAND}            { return TKAMPERSAND; }

{TKLB}                   { return TKLB; }

{TKLP}               { return TKLP; }
{TKRP}               { return TKRP; }
{TKLC}               { return TKLC; }
{TKRC}               { return TKRC; }
{TKRB}               { return TKRB; }

{TKCOMMA}            { return TKCOMMA; }
{TKQUESTIONMARK}     { return TKQUESTIONMARK; }
{TKCOLON}            { return TKCOLON; }
{TKSEMI}             { return TKSEMI; }
{TKTILDE}            { return TKTILDE; }

{ABA_TOK_DIGIT}+     { aba_yylval.ival = atoi(aba_yytext); return TKNUMBER; }

{ABA_TOK_ALPHA}({ABA_TOK_ALPHANUM})*  { aba_yylval.idname=strdup(aba_yytext); return TKATOM; }
"$"{ABA_TOK_ALPHANUM}+ { aba_yylval.idname=strdup(aba_yytext); return TKSTATENAME; }

{TKDOT}              { return TKDOT; }

<<EOF>>              { return TKEOF; }

.                    { aba_yyerror("Invalid character: %s", aba_yytext); }

%%

static int aba_skip_comment() 
{
  register int c;

  do { c = input(); } while ( c != '\n' && c != EOF );
  return 0;
}
